from my_app_name.common.util.parser import (
    create_default_filter_param_parser,
    create_default_sort_param_parser,
)

parse_filter_param = create_default_filter_param_parser()
parse_sort_param = create_default_sort_param_parser()
